#-------------------------------------------------------------------------------
# expandGO.pl
# May 15, 2016
# Read in file of GO terms and isas, add isas-of-isas
#-------------------------------------------------------------------------------
$GOFile = 'GOTerms.txt';
$expandedFile = 'expandedGOterms.txt';
#-------------------------------------------------------------------------------
%GOTerms = ();
$startTime = time ();
loadGOterms (); # load & parse file that provides text description for each GO id
$elapsedTime = time () - $startTime;
print "$elapsedTime seconds\n";

%expGOTerms = ();
findIsas ();
$elapsedTime = time () - $startTime;
print "$elapsedTime seconds\n";
writeExpandedGOs ();
$elapsedTime = time () - $startTime;
print "$elapsedTime seconds\n";
#-------------------------------------------------------------------------------
sub loadGOterms {

    print "loading GO terms\n";
    open (GO, $GOFile) || die;
    
    while (my $line = <GO>) {
        
        chomp ($line);
        my ($id, $namespace, $name, @isas) = split(/\t/, $line);
        $GOTerms {$id} {'namespace'} = $namespace;
        $GOTerms {$id} {'name'} = $name;
        push (@{$GOTerms{$id} {'isas'}}, @isas);
    }
    printf "%d GO terms loaded\n", scalar keys %GOTerms;
    close GO;

}
#-------------------------------------------------------------------------------
sub findIsas {
    
    print "Finding all all parent isas for each go term\n";
    
    $idCount = 0;
    $totalIsas = 0;
  
    foreach $id (sort keys %GOTerms) {
        
        my %newIsas = ();
        expandIsas ($id, \%newIsas);
        @{$expGOTerms {$id}} = keys %newIsas;
        $totalIsas += scalar keys %newIsas;
        $idCount++;
        if ($idCount % 1000 == 0) {
            printf "%d\t%1.3f\n", $idCount, $totalIsas / $idCount;
        }
    }
}
#-------------------------------------------------------------------------------
sub expandIsas {
    
    my $id = $_[0];
    my $newIsasRef = $_[1];
    my @isas = @{$GOTerms {$id} {'isas'}};
    
    foreach $id (@isas) {
       $newIsasRef->{$id} += 1;
       expandIsas($id, $newIsasRef);
    }
    
    return;
}
#-------------------------------------------------------------------------------
sub writeExpandedGOs {
    
    printf "Writing expanded GO terms to $expandedFile\n";
    open (OUT, ">$expandedFile") || die;
    
    foreach $id (sort keys %GOTerms) {
        printf OUT "%s\t%s\t%s", $id, $GOTerms {$id} {'namespace'}, $GOTerms {$id} {'name'};
        foreach $isa (sort @{$expGOTerms {$id}}) {
            printf OUT "\t%s", $isa;
        }
        print OUT "\n";
    }
}
 
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
