#-------------------------------------------------------------------------------
# selectPickLongest.pl
# May 11, 2016
#-------------------------------------------------------------------------------
use Bio::SeqIO;
#-------------------------------------------------------------------------------
$asmFile = 'E:\DBs\Csap\Transcriptome\CsapTrinityAsm2.fasta';
$selectedComponentsFile = 'components_SNP_assay_list.txt';
$outFile = 'selectToBlast.txt';
#-------------------------------------------------------------------------------
# Read in components selected to be blasted after longest sequence found
open (SLC, $selectedComponentsFile)|| die "$selectComponentsFile can't be opened";
@selectComponents = ();
while ($line = <SLC>) {
	chomp $line;
	($comp) = $line =~ /comp(\d+_c\d+)/;
	push (@selectComponents, $comp);
}
printf "%d components selected for blast searches\n", scalar @selectComponents;
#-------------------------------------------------------------------------------
# Create hash with arrays of sequences keyed to each component
%isoforms = ();
my $in = Bio::SeqIO->new(-file => $asmFile, -format => Fasta );
$n_sequences = 0;
while ($sequence = $in->next_seq()) {
	
	$n_sequences++;
 	($comp) = $sequence->id =~ /comp(\d+_c\d+)/;
	push (@{$isoforms {$comp}}, $sequence);
}
printf "%d sequences found for %d components\n",
	$n_sequences, scalar keys %isoforms;
#-------------------------------------------------------------------------------
# Pick the longest sequence for each selected component, print out sequence id
open (OUT, ">$outFile") || die;
foreach $comp (@selectComponents) {
	
	if (!exists $isoforms {$comp}) {
		print "Warning: No sequences found for selected component $comp\n";
	}
	if (scalar @{$isoforms {$comp}} > 1) {
		my @sorted = sort {$b->length <=> $a->length} @{$isoforms {$comp}};
		$seq = $sorted [0];
	}
	else {$seq = ${$isoforms {$comp}} [0];}
	printf OUT "%s\t%d\n", $seq->display_id, $seq->length;
}
close OUT;
print "Finished!\n";
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

