#-------------------------------------------------------------------------------
#!/usr/bin/perl -w
#-------------------------------------------------------------------------------
# selectProtBlast.pl
# BLAST components used for SNP assay against uniprot in transcriptome against uniprot to determine which components can be
# annotated. List of components are ones that passed Sophie's SNP screenings
# May 10, 2016
# J. Neigel
#-------------------------------------------------------------------------------
use Bio::SeqIO;
#-------------------------------------------------------------------------------
print "Running selectProtBlast.pl\n";
$db1 = 'E:\DBs\Uniprot\uniprot_sprot';
$db2 = 'E:\DBs\Uniprot\uniprot_trembl';
$assembly = 'E:/DBs/Csap/Transcriptome/CsapTrinityAsm2.fasta';
$inFile = 'components_SNP_assay_list.txt';
$outFile = 'components_SNP_assay_hits.txt';
#-------------------------------------------------------------------------------
@fieldNames = qw(qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore);
$header = join("\t", @fieldNames);
print OUT "$header\n";
#-------------------------------------------------------------------------------
# Start timer
$startTime = time ();
# Read transcriptome assembly, create id-to-seq hash
$in = Bio::SeqIO->new(-file => $assembly, -format => Fasta );
%idToTranscript = ();
# Process each sequence innput file
while ($seq = $in->next_seq()) {$idToTranscript {$seq->display_id} = $seq->seq;}
$elapsedTime = time () - $startTime;
print "transcriptome read in $elapsedTime seconds\n";
#-------------------------------------------------------------------------------
# Count total lines in file
open (IN, $inFile) || die;
$total = 0;
while ($in = <IN>) {$total++;}
close IN;
print "$total components to blast\n";
#-------------------------------------------------------------------------------
# Read in transcriptome sequence id, retrieve sequence
# Open files
open (IN, $inFile) || die;
open (OUT, ">$outFile") || die;
# start timer
$startTime = time ();
$count = 0;
$hitCount = 0;

while ($id = <IN>) {
	
	chomp $id;
	printf "Blasting query number %d: %s (%1.f percent done)\n",
		$count, $id, $count / $total;
	
	# Write sequence to temporary file for blast program
	open (TI, ">tempIn");
	printf TI ">%s\n%s\n", $id, $idToTranscript {$id};
	close (TI);
	
	$outFormat = '-outfmt "6"';

	# First try swissprot
	system ("blastx -query tempIn -db $db1 -out tmpOut $outFormat -num_threads 8");
   
	# open blast report and find out what happened
    open (TO, 'tmpOut');
	$topHit1 = <TO>;	# first line is  top hit, or nothing if no hit
	if (length $topHit1 < 2) {
		
		# Try again with trembl
		close TO;
		system ("blastx -query tempIn -db $db2 -out tmpOut $outFormat -num_threads 8");
		open (TO, 'tmpOut');
		$topHit2 = <TO>;	
		if (length $topHit2 < 2) {
			print OUT "$id\tNo Hits\n";
		}
		else {
			print OUT $topHit2;
			$hitCount++;
		}
	}
	else {
		print OUT $topHit1;
		$hitCount++;
	}
    close TO;
	
	$count += 1;
	#if ($count > 2) {
	#	last;
	#}
}
#-------------------------------------------------------------------------------
$elapsedTime = time () - $startTime;
print "protBlast.pl finished\n";
print "$hitCount hits\n";
printf "%d sequences blasted in %d hours %d minutes %d seconds\n",
    $count, $elapsedTime / 3600, ($elapsedTime % 3600) / 60,
    ($elapsedTime % 3600) % 60; 
printf "Average of %1.3f sequences per second\n",
    $count / $elapsedTime; 

close OUT;
unlink 'tmpOut';
unlink 'tmpIn';
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
